/* Helicostat program V1.0                              */

/* Author : Philippe Kauffmann                          */

/* This is the helicostat definition code               */

#include "main.h"
#include <math.h>

// cration de l'hlicostat et son dessin
int creerhelicostat(void)
{
// CAUTION : mass can never be 0 nor negative
helicostat = raydium_ode_object_create("helicostat");
ballon = raydium_ode_object_sphere_add("ballon",helicostat,p.M_BALLON+p.M_AIR,R_BALLON, \
            RAYDIUM_ODE_STANDARD,0,"ballon.tri");
nacelle = raydium_ode_object_box_add("nacelle",helicostat,p.M_NACELLE,L_NACELLE, \
            L_NACELLE,H_NACELLE,RAYDIUM_ODE_STANDARD,0,"nacelle.tri");
helice1 = raydium_ode_object_box_add("helice1",helicostat,0.0001,RAYDIUM_ODE_AUTODETECT, \
            RAYDIUM_ODE_AUTODETECT,RAYDIUM_ODE_AUTODETECT,RAYDIUM_ODE_STANDARD,0,"helice.tri");
helice2 = raydium_ode_object_box_add("helice2",helicostat,0.0001,RAYDIUM_ODE_AUTODETECT, \
            RAYDIUM_ODE_AUTODETECT,RAYDIUM_ODE_AUTODETECT,RAYDIUM_ODE_STANDARD,0,"helice.tri");

//Placement des lments
raydium_ode_element_move_3f(nacelle,0,0,POSITION);
raydium_ode_element_move_3f(ballon,0,0,POSITION+SEPARATION);
raydium_ode_element_move_3f(helice1,0,0,POSITION+0.18);
raydium_ode_element_move_3f(helice2,0,0,POSITION+0.04);

//attachement des lments ensembles
raydium_ode_joint_attach_fixed("jointprincipal",nacelle,ballon);
raydium_ode_joint_break_force_name("jointprincipal",300);
joint1 = raydium_ode_joint_attach_hinge("joint1",nacelle,helice1,0,0,POSITION+0.18,RAYDIUM_ODE_JOINT_AXE_Z);
joint2 = raydium_ode_joint_attach_hinge("joint2",nacelle,helice2,0,0,POSITION+0.06,RAYDIUM_ODE_JOINT_AXE_Z);

// ajout des moteurs
moteur1 = raydium_ode_motor_create("moteur1",helicostat,RAYDIUM_ODE_MOTOR_ENGINE);
raydium_ode_motor_attach(moteur1,joint1,0); // 0 is first and only axis
raydium_ode_motor_power_max(moteur1,100.0); // max speed
raydium_ode_motor_speed(moteur1,0.0);
moteur2 = raydium_ode_motor_create("moteur2",helicostat,RAYDIUM_ODE_MOTOR_ENGINE);
raydium_ode_motor_attach(moteur2,joint2,0); // 0 is first and only axis
raydium_ode_motor_power_max(moteur2,100.0); // max speed
raydium_ode_motor_speed(moteur2,-0.0);

return helicostat;
}

// aerodynamic force calculation and force applying
void aerodynHeli(int moteur)
{
// Donnes de forces et vitesses
dReal *vBallon, *vNacelle; // vitesses des lments en repre sol
dReal vrBallon[3], vrNacelle[3]; // vitesses des lments en repre hlicostat
dReal traineBallon, traineNacelle;
dReal fBallon[3], fNacelle[3]; // traines arodynamiques
dReal *oBallon, *oNacelle; // vitesses rotationnelle des lments en repre sol
dReal orBallon[3], orNacelle[3]; // vitesses rotationnelle des lments en repre hlicostat
int i;

// Pousse d'Archimde
raydium_ode_element_addforce_3f(ballon,0,0,p.M_AIR*GRAVITE); // Archimedes force
// Mise en flottaison
if(moteur) dBodyAddRelForce(raydium_ode_element[nacelle].body,0,0,(p.M_BALLON+p.M_NACELLE)*GRAVITE);

//Traine arodynamique en translation
// BALLON : calcul du module de vitesse du ballon
vBallon=raydium_ode_element_linearvelocity_get(ballon);
dBodyVectorFromWorld(raydium_ode_element[ballon].body,vBallon[0],vBallon[1],vBallon[2],vrBallon);
modVBallon = 0;
//Speed module computation
for(i=0;i<3;i++) modVBallon += vrBallon[i]*vrBallon[i];
modVBallon = sqrtf(modVBallon);
for(i=0;i<3;i++) fBallon[i] = (-p.T_BALLON_T)*vrBallon[i];
//On applique la traine au ballon
dBodyAddRelForce(raydium_ode_element[ballon].body,fBallon[0],fBallon[1],fBallon[2]);

// NACELLE : calcul du module de vitesse du nacelle
vNacelle=raydium_ode_element_linearvelocity_get(nacelle);
dBodyVectorFromWorld(raydium_ode_element[nacelle].body,vNacelle[0],vNacelle[1],vNacelle[2],vrNacelle);
for(i=0;i<3;i++) fNacelle[i] = (-p.T_NACELLE_T)*vrNacelle[i];
//On applique la traine  la nacelle
dBodyAddRelForce(raydium_ode_element[nacelle].body,fNacelle[0],fNacelle[1],fNacelle[2]);

// Traine arodynamique en rotation
// Application d'une traine visceuse au ballon
oBallon=raydium_ode_element_angularvelocity_get(ballon);
dBodyVectorFromWorld(raydium_ode_element[ballon].body,oBallon[0],oBallon[1],oBallon[2],orBallon);
dBodyAddRelTorque(raydium_ode_element[ballon].body,-p.T_BALLON_R*orBallon[0],-p.T_BALLON_R*orBallon[1],-p.T_BALLON_R*orBallon[2]);
// Application d'une traine visceuse au nacelle
oNacelle=raydium_ode_element_angularvelocity_get(nacelle);
dBodyVectorFromWorld(raydium_ode_element[nacelle].body,oNacelle[0],oNacelle[1],oNacelle[2],orNacelle);
dBodyAddRelTorque(raydium_ode_element[nacelle].body,-p.T_NACELLE_R*orNacelle[0],-p.T_NACELLE_R*orNacelle[1],-p.T_NACELLE_R*orNacelle[2]);
}
