/* Helicostat program V1.0                              */

/* Author : Philippe Kauffmann                          */

/* This is the main code of the helicostat simulator    */

#include "main.h"

//Fonctions
void step(void);
void display(void);

//Donnes globales
int moteur = 0;     // boolen de marche/arrt moteur
int zoom = 30;      // dfinition de l'angle visuel
// Sound variables
dReal soundPitch = 1.0, soundGain = 0.2;

// simulation physique
void step(void)
{
static dReal av_ar = 0, gche_drte = 0;

// Calcul des forces arodynamiques
aerodynHeli(moteur);
// Dplacement au joystick
if (raydium_joy_n_axes && moteur) {
    dBodyAddRelForce(raydium_ode_element[nacelle].body, 0, 0, -1.2*raydium_joy_axis[jc.JHB]);    // vertical axis
    soundPitch = 1-0.3*raydium_joy_axis[1]; // sound pitch adapted to power
    raydium_sound_SetSourcePitch(sonMoteurs,soundPitch);
    dBodyAddRelTorque(raydium_ode_element[nacelle].body, 0, 0, -0.2*raydium_joy_axis[jc.JTGD]);   // lacet
    dBodyAddRelTorque(raydium_ode_element[nacelle].body, 0, -0.6*raydium_joy_axis[jc.JAA], 0);   // tangage
    dBodyAddRelForce(raydium_ode_element[ballon].body, 0, -1.2*raydium_joy_axis[jc.JGD], 0);      // force latrale
    }
if (!raydium_joy_n_axes && moteur) {
// Dplacement au clavier
    switch(direction) {
        // moment appliqu au niveau du nacelle sur l'axe y
        case AVANT : if(av_ar<0.2) av_ar+= 0.2/100;break;
        case ARRIERE : if(av_ar>-0.2) av_ar -= 0.2/100;break;
        // pousse applique sur le ballon sur l'axe y
        case GAUCHE :  if(gche_drte<0.6) gche_drte += 0.6/100;break;
        case DROITE :  if(gche_drte>-0.6) gche_drte -= 0.6/100;break;
        // augmentation de la pouss des deux moteurs
        case HAUT :  dBodyAddRelForce(raydium_ode_element[nacelle].body, 0, 0, 1);break;
        case BAS :  dBodyAddRelForce(raydium_ode_element[nacelle].body, 0, 0, -1);break;
        // variation du couple contrarotatif des moteurs
        case TGAUCHE : dBodyAddRelTorque(raydium_ode_element[nacelle].body, 0, 0, 0.05);break;
        case TDROITE : dBodyAddRelTorque(raydium_ode_element[nacelle].body, 0, 0, -0.05);break;
        case STOP : if(av_ar>0) av_ar -= 0.2/100;if(av_ar<0) av_ar+= 0.2/100;
                    if(gche_drte>0) gche_drte -= 0.6/100;if(gche_drte<0) gche_drte+= 0.6/100;
        }
    dBodyAddRelTorque(raydium_ode_element[nacelle].body, 0, av_ar, 0);
    dBodyAddRelForce(raydium_ode_element[ballon].body, 0, gche_drte, 0);
    }
}

// affichage
void display(void)
{
dReal *pos, *angle;
// Orientation de la camra
int typeCam;

// gestion des touches du joystick
if (raydium_joy_n_axes) joyCom();
// gestion du clavier et dplacement de l'hlicostat
typeCam = keyCom();
// clear all hardware buffer
raydium_clear_frame();
// Orientation de la camra
switch(typeCam) {
    case GLUT_KEY_F2 :  pos=raydium_ode_element_pos_get(ballon); // chase helicopter view
                        angle=dBodyGetRotation(raydium_ode_element[ballon].body);
                        raydium_camera_smooth(pos[0]-5*angle[0],pos[1]+5*angle[1],pos[2]+0.1,pos[1],-pos[2],pos[0],\
                        30,0,raydium_frame_time*3);break;
    case GLUT_KEY_F3 :  raydium_window_view_perspective(70, 0.1, 1000); // define field view
                        raydium_ode_element_camera_inboard(nacelle,0.1,0,0.2,2,0,0);break; //  FPV
    default :           pos=raydium_ode_element_pos_get(ballon); // ground view
                        raydium_camera_smooth(-5,0,1.5,pos[1],-pos[2],pos[0],zoom,0,raydium_frame_time*3);
    }
raydium_ode_draw_all(RAYDIUM_ODE_DRAW_NORMAL);
//if(raydium_key[GLUT_KEY_F4]) raydium_ode_draw_all(RAYDIUM_ODE_DRAW_DEBUG); // show boxes
// Affichage des donnes  l'cran
affichTexte();
raydium_rendering_finish();
}

int main(int argc, char **argv)
{
raydium_init_args(argc,argv);
raydium_path_add("../modeles");
// function to load control parameters
loadControl();
switch (jc.ECRAN) {
    case 2 : raydium_window_create(800,600,RAYDIUM_RENDERING_WINDOW,"Helicostat d'Oemichen"); // working in window
             break;
    case 3 : raydium_window_create(1024,768,RAYDIUM_RENDERING_WINDOW,"Helicostat d'Oemichen"); // working in window
             break;
    case 4 : raydium_window_create(640,480,RAYDIUM_RENDERING_FULLSCREEN,"Helicostat d'Oemichen"); // working in window
             break;
    default : raydium_window_create(640,480,RAYDIUM_RENDERING_WINDOW,"Helicostat d'Oemichen"); // working in window
    }
// adapt frequency & gravity to obtain correct free fall time with distance unity = meter
// and force in Newton
raydium_ode_set_timestep(0.01);
raydium_ode_set_physics_freq(100.0);
raydium_ode_gravity_3f(0,0,-GRAVITE);

// bilinear filter gives some black pixels
raydium_texture_filter_change(RAYDIUM_TEXTURE_FILTER_TRILINEAR);
// define FOV, min and max display distance
raydium_window_view_perspective(70,0.1,1000);
// no fog to see sky
raydium_fog_disable();
// light management
raydium_light_enable();
raydium_light_on(0);        // start light number 0
raydium_light_conf_7f(0,50,150,200,1000000,1,0.9,0.9); // id, pos, intensity and color (RGB)
raydium_background_color_change(1,0.9,0.9,1);
// zone from camera in which to calculate shadows
raydium_shadow_camerabox_size(50);
raydium_shadow_enable();
// management on what is to see
raydium_mouse_show();           // don't hide mouse cursor
raydium_sky_box_cache();
// chargement de la scne
raydium_path_add("../scenes");
raydium_ode_ground_set_name("cocorobix.tri");
// load helicostat parameters
raydium_path_add("../modeles");
loadParam();
helicostat = creerhelicostat(); // display helicostat
if (!helicostat) exit(-3);
// Add sound
sonMoteurs=raydium_sound_LoadWav("engine.wav");
raydium_sound_SourcePlay(sonMoteurs);
raydium_sound_SetSourcePitch(sonMoteurs,1.0);
raydium_sound_SetSourceGain(sonMoteurs,0.0);
// raydium and ODE infinite loops
raydium_ode_StepCallback=step;
// raydium display 50 Hz step max
raydium_render_fps_limit(50);
raydium_callback(&display);

exit(0);
}
