/* Helicostat program V1.0                              */

/* Author : Philippe Kauffmann                          */

/* This is the main header of the helicostat simulator  */

#ifndef HELICOSTAT_H
#define HELICOSTAT_H

#include <stdio.h>
#include "raydium/index.c"

#define GRAVITE 9.81        // gravit terrestre en units SI

// fixed moobile parameters
#define R_BALLON    0.39      // rayon inertiel quivalent du ballon en m
#define L_NACELLE   0.3       // cot inertiel quivalent de la nacelle en m
#define H_NACELLE   0.1       // hauteur inertielle quivalente de la nacelle en m
#define SEPARATION  0.505     // distance entre CM ballon et CM nacelle en m
#define POSITION    -1.24     // position au-dessus du sol de l'hlicostat en m

// variable mobile parameters
struct {
dReal M_BALLON;             // masse du ballon en kg
dReal M_AIR;                // masse de l'air contenu dans le ballon en kg
dReal M_NACELLE;            // masse de la nacelle en kg
dReal T_BALLON_T;           // coefficient de traine en sub-critique du ballon
dReal T_NACELLE_T;          // coefficient de traine en sub-critique de la nacelle
dReal T_BALLON_R;           // coefficient de traine visqueuse en rotation du ballon
dReal T_NACELLE_R;          // coefficient de traine visqueuse en rotation du nacelle */
}p;

// program, keyboard and joystick parameters
struct {
int ECRAN;              // taille d'acran de dmarrage
int CHAUT;       	    // touche z pour monter
int CBAS;        		// touche x pour descendre
int CAVANT;      		// touche o pour avancer
int CARRIERE;    		// touche : pour reculer
int JHB;                // axe haut/bas du joystick
int JAA;                // axe avant/arrire du joystick
int JGD;                // axe de roulis du joystick
int JTGD;               // axe de lacet du joystick
int BSS;                // bouton start/stop
int BNH;                // bouton nouvel hlicostat
int BZP;                // bouton zoom plus
int BZM;                // bouton zoom moins
int BVP;                // bouton volume plus
int BVM;                // bouton volume moins
}jc;

// Directions de mouvement
enum {STOP,GAUCHE,DROITE,AVANT,ARRIERE,HAUT,BAS,TGAUCHE,TDROITE} direction;
// lments et objets
int helicostat, ballon, nacelle, helice1, helice2, moteur1, moteur2, joint1, joint2;
 // Modules des vitesses du ballon et de la nacelle
dReal modVBallon, modVNacelle;
// pointeur sur le systme de son
int sonMoteurs;

/// Fonctions
// cration d'un hlicostat
int creerhelicostat(void);
// chargement depuis un fichier des paramtres de l'hlicostat
void loadParam(void);
// chargement depuis un fichier des paramtres de joystick et clavier
void loadControl(void);
// calcul des forces sur l'hlicostat
void aerodynHeli(int moteur);
// gestion des boutons du joystick
void joyCom(void);
// gestion du clavier
int keyCom(void);
// affichage de texte  l'cran
void affichTexte(void);
#endif
